/* This is "lifter.c", written to accompany Introduction to
   Scientific Programming by Joseph L. Zachary .*/

/* This program solves the kinematics of a weightlifting robot
   as described in Chapter 11 of the text. */


#include <stdio.h>
#include <math.h>



/* We obtain the knee angle in degrees and the torso length in
   millimeters of the robot.  We then apply the model and assumptions
   from Chapter 11 to calculate and report the lengths of the torso,
   thigh, and shin; the angle and position of the ankle; the angle and
   position of the knee; the angle and position of the hip; and the
   position of the shoulder. */

void main (void)
{

  int shin,           /* The lengths of the shin, thigh, and */
      thigh,          /* torso in millimeters. */
      torso;

  double kneeX,       /* The x-y coordinates of the knee and hip, and */
         kneeY,       /* the y-coordinate of the shoulder.  The origin */
         hipX,        /* of the coordinate system is at the ankle, and */
         hipY,        /* the x-coordinate of the shoulder is */
         shoulderY;   /* constrained to be zero. */

  double ankleAngle,  /* The angles in degrees formed by ground and */
         kneeAngle,   /* shin (ankle), shin and thigh (knee), and thigh */
         hipAngle;    /* and torso (hip). */
  
  double alpha,       /* The measurements in radians of the angles from */
         beta,        /* Figure 11.2 of the text. */
         gamma;

  double pi;          /* The value of pi. */


  /* We begin by prompting for "kneeAngle" and "torso". */
  
  printf("Enter knee angle (deg): ");
  scanf("%lf", &kneeAngle);

  printf("Enter torso length (mm): ");
  scanf("%d", &torso);


  /* We calculate pi using arccos.  This works because cos(pi) = -1. */

  pi = acos(-1);


  /* We assumed these relationships when developing the model in
     Chapter 11. */

  ankleAngle = kneeAngle / 2;
  thigh = 0.67 * torso;
  shin = 0.59 * torso;


  /* We calculate "alpha" and the knee coordinates ... */

  alpha = (ankleAngle / 180) * pi;
  kneeX = shin * cos(alpha);
  kneeY = shin * sin(alpha);

  /* ... "beta" and the hip coordinates ... */

  beta = (kneeAngle / 180) * pi - alpha;
  hipX = kneeX - thigh * cos(beta);
  hipY = kneeY + thigh * sin(beta);

  /* ... and "gamma", "hipAngle", and the shoulder coordinate. */

  gamma = acos(-hipX/torso);
  hipAngle = ((gamma + beta) / pi) * 180;
  shoulderY = hipY + torso * sin(gamma);


  /* Finally, we display our results. */

  printf("\nTorso = %d mm\n", torso);
  printf("Thigh = %d mm\n", thigh);
  printf("Shin  = %d mm\n", shin);

  printf("\nAnkle angle = %g deg\n", ankleAngle);
  printf("          X = %g mm\n", 0.0);
  printf("          Y = %g mm\n", 0.0);

  printf("\nKnee angle = %g deg\n", kneeAngle);
  printf("         X = %g mm\n", kneeX);
  printf("         Y = %g mm\n", kneeY);

  printf("\nHip angle = %g deg\n", hipAngle);
  printf("        X = %g mm\n", hipX);
  printf("        Y = %g mm\n", hipY);

  printf("\nShoulder X = %g mm\n", 0.0);
  printf("         Y = %g mm\n", shoulderY);

}
